/*
 * Project name:
     color7_mm_example.mcpar
 * Generated by:
     Visual TFT
 * Date of creation
     12.6.2018.
 * Test configuration:
     MCU:             STM32F107VC
     Dev.Board:       EasyMx_PRO_v7_for_STM32_ARM_9A
     Oscillator:      72000000 Hz
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 */

#include "color7_mm_example_objects.h"
#include "color7_mm_example_resources.h"
#include "Click_Color7_types.h"
#include "Click_Color7_config.h"

uint8_t isColor = 0;
float colorValue;
char hslData_text[ 20 ]   = "0.00";
char redData_text[ 20 ]   = "0.00";
char greenData_text[ 20 ] = "0.00";
char blueData_text[ 20 ]  = "0.00";

float redRatio    = 0;
float greenRatio = 0;
float blueRatio   = 0;

void updateLabel(TLabel *lbl, char *str)
{
    TFT_Set_Font(lbl->FontName, Screen1.Color, FO_HORIZONTAL);
    TFT_Write_Text(lbl->Caption, lbl->Left, lbl->Top);
    strcpy(lbl->Caption, str);
    DrawLabel(lbl);
}

void systemInit()
{
    mikrobus_gpioInit( _MIKROBUS1, _MIKROBUS_INT_PIN, _GPIO_INPUT );
    mikrobus_i2cInit( _MIKROBUS1, &_COLOR7_I2C_CFG[0] );
    mikrobus_logInit( _LOG_USBUART_A, 9600 );
    mikrobus_logWrite("--- System init --- ", _LOG_LINE);
    Delay_ms( 100 );
}

void applicationInit()
{
    color7_i2cDriverInit( (T_COLOR7_P)&_MIKROBUS1_GPIO, (T_COLOR7_P)&_MIKROBUS1_I2C, 0x29 );
    mikrobus_logWrite("--- Color Init --- ", _LOG_LINE);
    color7_writeByte(_COLOR7_REG_ENABLE, _COLOR7_ENABLE_RGBC_INTERRUPT |
                                         _COLOR7_ENABLE_WAIT |
                                         _COLOR7_ENABLE_RGBC |
                                         _COLOR7_POWER_ON );

    color7_writeByte(_COLOR7_REG_CONTROL, _COLOR7_CTRL_GAIN_x16);
    color7_writeByte(_COLOR7_REG_RGBC_TIME, _COLOR7_RGBC_TIME_700ms);
    mikrobus_logWrite("--- Start measurement --- ", _LOG_LINE);

    DrawCircle(&Color_circle, 0x0008);
}

void applicationTask()
{
    TFT_Set_Font(&Tahoma12x16_Regular, 0xFFFF, FO_HORIZONTAL);
    TFT_Write_Text(hslData_text, 171, 186);
    colorValue = color7_getColorValue();
    FloatToStr(colorValue, hslData_text);
    TFT_Set_Font(&Tahoma12x16_Regular, 0x0000, FO_HORIZONTAL);
    TFT_Write_Text(hslData_text, 171, 186);
    
    TFT_Set_Font(&Tahoma12x16_Regular, 0xFFFF, FO_HORIZONTAL);
    TFT_Write_Text(redData_text, 13, 59);
    redRatio = color7_readColorRatio(_COLOR7_COLOR_DATA_RED);
    FloatToStr(redRatio, redData_text);
    TFT_Set_Font(&Tahoma12x16_Regular, 0x0000, FO_HORIZONTAL);
    TFT_Write_Text(redData_text, 13, 59);

    TFT_Set_Font(&Tahoma12x16_Regular, 0xFFFF, FO_HORIZONTAL);
    TFT_Write_Text(greenData_text, 15, 115);
    greenRatio = color7_readColorRatio(_COLOR7_COLOR_DATA_GREEN);
    FloatToStr(greenRatio, greenData_text);
    TFT_Set_Font(&Tahoma12x16_Regular, 0x0000, FO_HORIZONTAL);
    TFT_Write_Text(greenData_text, 15, 115);
    
    TFT_Set_Font(&Tahoma12x16_Regular, 0xFFFF, FO_HORIZONTAL);
    TFT_Write_Text(blueData_text, 14, 173);
    blueRatio = color7_readColorRatio(_COLOR7_COLOR_DATA_BLUE);
    FloatToStr(blueRatio, blueData_text);
    TFT_Set_Font(&Tahoma12x16_Regular, 0x0000, FO_HORIZONTAL);
    TFT_Write_Text(blueData_text, 14, 173);
    
    isColor = color7_getColor(colorValue);
    switch(isColor)
    {
        case 1:
        {
            mikrobus_logWrite("--- Color: ORANGE ", _LOG_LINE);
            DrawCircle(&Color_circle, 0xF000);
            break;
        }
        case 2:
        {
            mikrobus_logWrite("--- Color: RED ", _LOG_LINE);
            DrawCircle(&Color_circle, 0xF800);
            break;
        }
        case 3:
        {
            mikrobus_logWrite("--- Color: PINK ", _LOG_LINE);
            DrawCircle(&Color_circle, 0xF00F);
            break;
        }
        case 4:
        {
            mikrobus_logWrite("--- Color: PURPLE ", _LOG_LINE);
            DrawCircle(&Color_circle, 0xF81F);
            break;
        }
        case 5:
        {
            mikrobus_logWrite("--- Color: BLUE ", _LOG_LINE);
            DrawCircle(&Color_circle, 0x001F);
            break;
        }
        case 6:
        {
            mikrobus_logWrite("--- Color: CYAN ", _LOG_LINE);
            break;
        }
        case 7:
        {
            mikrobus_logWrite("--- Color: GREEN ", _LOG_LINE);
            DrawCircle(&Color_circle, 0x34C0);
            break;
        }
        case 8:
        {
            mikrobus_logWrite("--- Color: YELLOW ", _LOG_LINE);
            break;
        }
        default:
        {
            break;
        }
    }
    Delay_ms( 500 );
}

void main()
{
  Start_TP();
    systemInit();
    applicationInit();

  while (1) {
    //Check_TP();
        applicationTask();
  }
}
