#include "color7_mm_example_objects.h"
#include "color7_mm_example_resources.h"
#include "demo_types.h"
#include "demo_config.h"

float isColor;
float colorValue;
char hslData_text  [ 20 ]   = "0.00";
char redData_text  [ 20 ]   = "0.00";
char greenData_text[ 20 ]   = "0.00";
char blueData_text [ 20 ]   = "0.00";

float redRatio    = 0;
float greenRatio  = 0;
float blueRatio   = 0;

void systemInit()
{
    mikrobus_gpioInit( _MIKROBUS1, _MIKROBUS_INT_PIN, _GPIO_INPUT );
    mikrobus_i2cInit( _MIKROBUS1, &_COLOR7_I2C_CFG[0] );
    mikrobus_i2cInit( _MIKROBUS2, &_LEDDRIVER3_I2C_CFG[0] );
    mikrobus_logInit( _LOG_USBUART_A, 9600 );
    mikrobus_logWrite("--- System init --- ", _LOG_LINE);
    Delay_ms( 100 );
}

void applicationInit()
{
    color7_i2cDriverInit( (T_COLOR7_P)&_MIKROBUS1_GPIO, (T_COLOR7_P)&_MIKROBUS1_I2C, 0x29 );
    leddriver3_i2cDriverInit( (T_LEDDRIVER3_P)&_MIKROBUS2_GPIO, (T_LEDDRIVER3_P)&_MIKROBUS2_I2C, 0x38 );

    color7_writeByte(_COLOR7_REG_ENABLE, _COLOR7_ENABLE_RGBC_INTERRUPT |
                                         _COLOR7_ENABLE_WAIT |
                                         _COLOR7_ENABLE_RGBC |
                                         _COLOR7_POWER_ON );

    color7_writeByte(_COLOR7_REG_CONTROL, _COLOR7_CTRL_GAIN_x1);
    color7_writeByte(_COLOR7_REG_RGBC_TIME, _COLOR7_RGBC_TIME_700ms);
    
    leddriver3_setIntensity(_LEDDRIVER3_CONSTANT | _LEDDRIVER3_INTENSITY_16);
    mikrobus_logWrite("--- Start measurement --- ", _LOG_LINE);
}

void applicationTask()
{
    TFT_Set_Font(&Tahoma12x16_Regular, 0xFFFF, FO_HORIZONTAL);
    TFT_Write_Text(hslData_text, 171, 186);
    colorValue = color7_getColorValue();
    isColor = colorValue;
    colorValue *= 100.00;
    FloatToStr(colorValue, hslData_text);
    TFT_Set_Font(&Tahoma12x16_Regular, 0x0000, FO_HORIZONTAL);
    TFT_Write_Text(hslData_text, 171, 186);
    
    TFT_Set_Font(&Tahoma12x16_Regular, 0xFFFF, FO_HORIZONTAL);
    TFT_Write_Text(redData_text, 13, 59);
    redRatio = color7_readColorRatio(_COLOR7_COLOR_DATA_RED);
    redRatio *= 100.00;
    FloatToStr(redRatio, redData_text);
    TFT_Set_Font(&Tahoma12x16_Regular, 0x0000, FO_HORIZONTAL);
    TFT_Write_Text(redData_text, 13, 59);

    TFT_Set_Font(&Tahoma12x16_Regular, 0xFFFF, FO_HORIZONTAL);
    TFT_Write_Text(greenData_text, 15, 115);
    greenRatio = color7_readColorRatio(_COLOR7_COLOR_DATA_GREEN);
    greenRatio *= 100.00;
    FloatToStr(greenRatio, greenData_text);
    TFT_Set_Font(&Tahoma12x16_Regular, 0x0000, FO_HORIZONTAL);
    TFT_Write_Text(greenData_text, 15, 115);
    
    TFT_Set_Font(&Tahoma12x16_Regular, 0xFFFF, FO_HORIZONTAL);
    TFT_Write_Text(blueData_text, 14, 173);
    blueRatio = color7_readColorRatio(_COLOR7_COLOR_DATA_BLUE);
    blueRatio *= 100.00;
    FloatToStr(blueRatio, blueData_text);
    TFT_Set_Font(&Tahoma12x16_Regular, 0x0000, FO_HORIZONTAL);
    TFT_Write_Text(blueData_text, 14, 173);
    
    if((isColor >= 0.992) && (isColor <= 0.999))
    {
        mikrobus_logWrite("--- Color: ORANGE ", _LOG_LINE);
        DrawCircle(&Color_circle, 0xF000);
        leddriver3_setColor(_LEDDRIVER3_COLOR_ORANGE);
    }
    else if ((isColor >= 0.9750) && (isColor <= 0.9919))
    {
        mikrobus_logWrite("--- Color: RED ", _LOG_LINE);
        DrawCircle(&Color_circle, 0xF800);
        leddriver3_setColor(_LEDDRIVER3_COLOR_RED);
    }
    else if ((isColor >= 0.920) && (isColor <= 0.9749))
    {
        mikrobus_logWrite("--- Color: PINK ", _LOG_LINE);
        DrawCircle(&Color_circle, 0xF00F);
        leddriver3_setRGB_Color(0x5F, 0x60, 0x90);;
    }
    else if ((isColor >= 0.6201) && (isColor <= 0.919))
    {
        mikrobus_logWrite("--- Color: PURPLE ", _LOG_LINE);
        DrawCircle(&Color_circle, 0xF81F);
        leddriver3_setColor(_LEDDRIVER3_COLOR_PURPLE);
    }
    else if ((isColor >= 0.521) && (isColor <= 0.6200))
    {
        mikrobus_logWrite("--- Color: BLUE ", _LOG_LINE);
        DrawCircle(&Color_circle, 0x001F);
        leddriver3_setColor(_LEDDRIVER3_COLOR_BLUE);
    }
    else if ((isColor >= 0.470) && (isColor < 0.520))
    {
        mikrobus_logWrite("--- Color: CYAN ", _LOG_LINE);
        DrawCircle(&Color_circle, 0x07FF);
        leddriver3_setRGB_Color(0x40, 0x7F, 0x9F);
    }
    else if ((isColor >= 0.210) && (isColor <= 0.469))
    {
        mikrobus_logWrite("--- Color: GREEN ", _LOG_LINE);
        DrawCircle(&Color_circle, 0x34C0);
        leddriver3_setColor(_LEDDRIVER3_COLOR_GREEN);
    }
    else if ((isColor >= 0.0650) && (isColor <= 0.180))
    {
        mikrobus_logWrite("--- Color: YELLOW ", _LOG_LINE);
        DrawCircle(&Color_circle, 0xFFE0);
        leddriver3_setColor(_LEDDRIVER3_COLOR_YELLOW);
    }
    else
    {
        mikrobus_logWrite("--- Color: WHITE ", _LOG_LINE);
        DrawCircle(&Color_circle, 0xFFFF);
        leddriver3_setColor(_LEDDRIVER3_COLOR_OFF);
    }
    Delay_ms( 200 );
}

void main()
{
    Start_TP();
    systemInit();
    applicationInit();

    while (1)
    {
        applicationTask();
    }
}